################################################################################
# Created       : Martin Foster
# Modified      : 07-Sep-2003
################################################################################
#
# Stamp - Module part of Ethereal Realms, designed to handle timestamps
# Copyright (C) 2000-2004  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 29-26503 TWP RD 511
#                 Spruce Grove, Alberta
#                 T7Y 1G4
#
#################################################################################

package Ethereal::Stamp;					# Package name

use strict;							# Strict handling


################################################################################
# Sub-Routines
################################################################################

#####################
# Constructor
#
# Allows to class to be exported and thus spawn objects as a result

sub new
{
	#####################
	# Data Members
	my $class    = shift;					# Self reference
	my $database = shift;					# Database handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template
	my $realm    = shift;					# Realm hash
	my $pref     = shift;					# Preffered position
	my $trgt     = shift;					# Target audience


	#####################
	# Program Area

	# Create internal hash
	my $self     = {
		DATE   => $database->GetDateFormat($param->{'ROOM'}),
		PARAM  => $param,
		PREF   => $pref,
		REALM  => $realm,
		TARGET => $trgt,
		TMPL   => $tmpl
 	 };

	# Bless $self into designated class
	bless $self, $class;

	# Return self
	return $self;
}


#####################
# Date/Time stamp
#
# Functionality that allows the host to decide more or less where the date and
# time stamp to be placed.   Depending on the locale, different output will
# be generated.
#
# Gets a hash from realm which uses the GetHashRealm method.  This comments
# added to bypass potential problems with tag cleaning.

sub Time
{
	#####################
	# Data members
	my $self     = shift;					# Self referencing link
	my $tag      = shift;					# String to handle
	my $title    = shift;					# String to handle

	my $param    = $self->{'PARAM'};			# Parameter handle
	my $pref     = $self->{'PREF'};				# Preffered position
	my $realm    = $self->{'REALM'};			# Realm hash
	my $trgt     = $self->{'TARGET'};			# Target audience
	my $time     = $self->{'DATE'};				# Preffered position
	my $tmpl     = $self->{'TMPL'};				# Template
	
	my $pesky;						# Handling of global posts


	#####################
	# Program area	

	# Determine audience
	# Simply the date for these
	unless (($trgt ne 'all') 
	    && ($trgt ne 'system')
	    && ($trgt ne 'dice roll')
	    && ($trgt ne $realm->{'TxtSystem'}))
	{
		# No need to change values
		if (($tag !~ /DATETIME/)
		 && ($title != /DATETIME/))
		{
			# Change format
			if ($pref eq 'bottom')
			{
				# Classical behaviour
				$tag .= $tmpl->Pass('TagPostTagline', MNAME=>'DATETIME');
			}
			elsif ($pref eq 'top')
			{
				# Alternate behaviour
				$title = $tmpl->Pass('TagPostTagline', MNAME=>'DATETIME') . $title;
			}
		}
	}

	# Date with identification
	else
	{
		# Initialize
		$pesky = '';
		
		# Change format
		# Generate regular string
		if ($pref eq 'bottom')
		{
			# Pesky handling
			$pesky = $tmpl->Pass('TagPostTagline', MNAME=>$realm->{'TagPuppetPMRealm'} . $param->{'ROOM'})
		          if (defined($param->{'PESKY'}));

			# Set string
			$tag =  $tag 
			     .  $tmpl->Pass('TagPostTagline', MNAME=>$realm->{'TagPuppetPMFrom'} . $param->{'CHAR'})
			     .  $tmpl->Pass('TagPostTagline', MNAME=>$realm->{'TagPuppetPMTo'} . $param->{'OTARGET'})
			     .  $pesky; 
				     
			# Append timestamp if required
			$tag .= $tmpl->Pass('TagPostTagline', MNAME=>'DATETIME')
			 if (($tag !~ /DATETIME/) && ($title != /DATETIME/));
		}

		# Generate stripped string
		else
		{
			# Pesky handling
			$pesky = $tmpl->Pass('TagPostTime', MNAME=>$realm->{'TagPuppetPMSep'} . $realm->{'TagPuppetPMRealm'} . $param->{'ROOM'})
			 if (defined($param->{'PESKY'}));

			# Set string
			$title =  $title 
			       .  $tmpl->Pass('TagPostTime', MNAME=>$realm->{'TagPuppetPMSep'} . $realm->{'TagPuppetPMFrom'} . $param->{'CHAR'})
			       .  $tmpl->Pass('TagPostTime', MNAME=>$realm->{'TagPuppetPMSep'} . $realm->{'TagPuppetPMTo'} . $param->{'OTARGET'})
			       .  $pesky; 

			# Append timestamp if required
			$title .= $tmpl->Pass('TagPostTtime', MNAME=>$realm->{'TagPuppetPMSep'} . 'DATETIME')
			 if (($tag !~ /DATETIME/) && ($title != /DATETIME/));
		}
	}

	# Final replacement
	$tag  =~ s/DATETIME/$time/gs;
	$title =~ s/DATETIME/$time/gs;

	# Return unchanged
	return $tag, $title;
}

1;								# Return true
