--
-- 3.6.0 TO 4.0.0 --
--
-- RUN THE COMMANDS BELOW AT THE COMMAND LINE
--
-- > ./pg_dump -h [PGHOST] -U [PGUSER] -a -t Rotate [PGDBNAME] -D -O -R > /tmp/data.sql
--
-- > pgql-U [DBNAME] -h [DBHOST] ethereal
--   [Paste in text below]

-- Ability to detect unread messages
ALTER TABLE ethereal.Mail ADD MailRead TIMESTAMP;



-- FUNCTION    : getDateRSS
--
-- DESCRIPTION : This will return a properly formatted Date stamp for use
--               in RSS feeds.

CREATE OR REPLACE FUNCTION ethereal.getDateRSS(TIMESTAMP) RETURNS VARCHAR AS'
  DECLARE
    varTimestamp ALIAS FOR $1;				-- Timestamp

  BEGIN

    -- Quick and simple
    RETURN TO_CHAR(varTimestamp, ''Dy, DD Mon YYYY HH24:MM:SS'');

  END;
 ' LANGUAGE 'plpgsql';


--
--
-- FUNCTION    : getDateRSS
--
-- DESCRIPTION : This will return a properly formatted Date stamp for use
--               in RSS feeds.

CREATE OR REPLACE FUNCTION ethereal.getDateRSS(TIMESTAMP WITH TIME ZONE) RETURNS VARCHAR AS'
  DECLARE
    varTimestamp ALIAS FOR $1;				-- Timestamp

  BEGIN

    -- Quick and simple
    RETURN TO_CHAR(varTimestamp, ''Dy, DD Mon YYYY HH24:MM:SS TZ'');

  END;
 ' LANGUAGE 'plpgsql';


-- Create new sequence
CREATE SEQUENCE ethereal.seqFeed     MAXVALUE 2147483647;
ALTER TABLE ethereal.seqFeed         OWNER TO ethereal;


--
-- NAME        : RSS Feeds
-- REFERENCES  : Realm
--
-- DESCRIPTION : RSS Feeds is designed to act as a storage area for preferences
--               and archives when dealing with active RSS Feeds for display.

CREATE TABLE ethereal.Feed (
  FeedID               INT           NOT NULL DEFAULT NEXTVAL('ethereal.seqFeed'),
  RealmName            VARCHAR(30)   NOT NULL,
  FeedTitle            VARCHAR(30)   NOT NULL,
  FeedLink             VARCHAR(250)  NOT NULL,
  FeedSource           VARCHAR(250)  NOT NULL,
  FeedMax              SMALLINT      NOT NULL DEFAULT 0,
  FeedCompactView      VARCHAR(3)    NOT NULL DEFAULT 'yes',
  FeedShowDate         VARCHAR(3)    NOT NULL DEFAULT 'yes',
  FeedNewWindow        VARCHAR(3)    NOT NULL DEFAULT 'yes',
  FeedArchive          TEXT,
  FeedTimestamp        TIMESTAMP     NOT NULL DEFAULT LOCALTIMESTAMP,
  CONSTRAINT enumFeedCompactView     CHECK (FeedCompactView IN ('yes','no')),
  CONSTRAINT enumFeedShowDate        CHECK (FeedShowDate    IN ('yes','no')),
  CONSTRAINT enumFeedNewWindow       CHECK (FeedNewWindow   IN ('yes','no')),
  CONSTRAINT pkFeed                  PRIMARY KEY (FeedID),
  CONSTRAINT fkFeed                  FOREIGN KEY (RealmName)
    REFERENCES ethereal.Realm (RealmName)
      ON DELETE CASCADE
      ON UPDATE CASCADE
) WITHOUT OIDS;

-- STANDARD INDEX
CREATE INDEX idxFeed_Realm ON ethereal.Feed
(
  RealmName
);

-- Change ownership
ALTER TABLE ethereal.Feed                  OWNER TO ethereal;

--
--
-- FUNCTION    : getCurrent
--
-- DESCRIPTION : A simple function that is designed to simply return an
--               integer to determine if the file is new.


CREATE OR REPLACE FUNCTION ethereal.getCurrent(TIME WITHOUT TIME ZONE) RETURNS INTEGER AS'
  DECLARE
    -- Alias
    varTimestamp ALIAS FOR $1;			-- Puppeteer Login

    -- Declarations
    varCurrent TIMESTAMP WITH TIME ZONE;

  BEGIN

    -- Pull current time
    varCurrent := ''now'';

    -- Which is newer
    IF varTimestamp > (varCurrent - INTERVAL ''20 Minutes'') THEN
      -- Expired
      RETURN 1;
    ELSE
      -- Active
      RETURN 0;
    END IF;

  END;
 ' LANGUAGE 'plpgsql';


--
--
-- FUNCTION    : getCurrent
--
-- DESCRIPTION : A simple function that is designed to simply return an
--               integer to determine if the file is new.

CREATE OR REPLACE FUNCTION ethereal.getCurrent(TIMESTAMP WITH TIME ZONE) RETURNS INTEGER AS'
  DECLARE
    -- Alias
    varTimestamp ALIAS FOR $1;			-- Puppeteer Login

    -- Declarations
    varCurrent TIMESTAMP WITH TIME ZONE;

  BEGIN

    -- Pull current time
    varCurrent := ''now'';

    -- Which is newer
    IF (varTimestamp > varCurrent - INTERVAL ''20 Minutes'') THEN
      -- Expired
      RETURN 1;
    ELSE
      -- Active
      RETURN 0;
    END IF;

  END;
 ' LANGUAGE 'plpgsql';



-- Drop table
DROP TABLE ethereal.Rotate;

--
--
-- NAME        : Rotate
-- REFERENCES  : NIL
--
-- DESCRIPTION : Rotate is an independant table that exists to generate various 
--               material that will be rotated as to provide an internal method for
--               varying displayed information.

CREATE TABLE ethereal.Rotate (
  RotateName           VARCHAR(25)    NOT NULL,
  RotateZone           VARCHAR(10)     NOT NULL DEFAULT 'banner',
  RotateConduct        VARCHAR(8)     NOT NULL DEFAULT 'static',
  RotateTimestamp      TIMESTAMP      NOT NULL DEFAULT LOCALTIMESTAMP,
  RotateData           TEXT,
  CONSTRAINT pkRotate                 PRIMARY KEY (RotateName, RotateZone),
  CONSTRAINT enumRotateZone           CHECK (RotateZone IN ('banner','gallery','javascript','portal','ssi')),
  CONSTRAINT enumRotateConduct        CHECK (RotateConduct IN ('image','static','text'))
) WITHOUT OIDS;

ALTER TABLE ethereal.Rotate                OWNER TO ethereal;

-- Re-insert information
\i /tmp/data.sql

-- Limitation row
ALTER TABLE ethereal.Puppeteer ADD PuppeteerSnub VARCHAR(3);
UPDATE ethereal.Puppeteer  SET PuppeteerSnub='no';
ALTER TABLE ethereal.Puppeteer  ALTER COLUMN PuppeteerSnub SET NOT NULL;
ALTER TABLE ethereal.Puppeteer  ALTER COLUMN PuppeteerSnub SET DEFAULT 'no';
ALTER TABLE ethereal.Puppeteer  ADD CONSTRAINT enumPuppeteerSnub CHECK (PuppeteerSnub IN ('yes','no'));

-- Harassment
ALTER TABLE ethereal.Realm ADD RealmHarass VARCHAR(3);
UPDATE ethereal.Realm  SET RealmHarass='no';
ALTER TABLE ethereal.Realm  ALTER COLUMN RealmHarass SET NOT NULL;
ALTER TABLE ethereal.Realm  ALTER COLUMN RealmHarass SET DEFAULT 'no';
ALTER TABLE ethereal.Realm  ADD CONSTRAINT enumRealmHarass CHECK (RealmHarass IN ('yes','no'));


-- Harassment
ALTER TABLE ethereal.Work ADD PuppeteerLogin VARCHAR(10);
ALTER TABLE ethereal.Work ALTER COLUMN PuppeteerLogin SET NOT NULL;
ALTER TABLE ethereal.Work ADD CONSTRAINT fkWork_Owner FOREIGN KEY (PuppeteerLogin)
    REFERENCES ethereal.Puppeteer (PuppeteerLogin)
      ON DELETE CASCADE
      ON UPDATE CASCADE;

CREATE INDEX idxWork_Ident ON ethereal.Work
(
  GalleryID,
  WorkID,
  PuppeteerLogin
);

--
--
-- NAME        : GalleryAccess
-- REFERENCES  : Gallery
--               Work
--               Puppeteer
--
-- DESCRIPTION : Comment allows users to make comments on works located under
--               galleries.
DROP TABLE ethereal.GalleryAccess;

CREATE TABLE ethereal.GalleryAccess (
  GalleryID            INT           NOT NULL,
  PuppeteerLogin       VARCHAR(10)   NOT NULL,
  PuppetName           VARCHAR(30)   NOT NULL,
  AccessOwner          VARCHAR(3)    NOT NULL DEFAULT 'no',
  AccessEditor         VARCHAR(3)    NOT NULL DEFAULT 'no',
  AccessJanitor        VARCHAR(3)    NOT NULL DEFAULT 'no',
  CONSTRAINT pkGalleryAccess         PRIMARY KEY (GalleryID, PuppeteerLogin),
  CONSTRAINT fkAccess_ID             FOREIGN KEY (GalleryID)
    REFERENCES ethereal.Gallery (GalleryID)
      ON DELETE CASCADE
      ON UPDATE CASCADE,
  CONSTRAINT fkAccess_Login          FOREIGN KEY (PuppeteerLogin)
    REFERENCES ethereal.Puppeteer (PuppeteerLogin)
      ON DELETE CASCADE
      ON UPDATE CASCADE,
  CONSTRAINT enumAccessOwner         CHECK (AccessOwner       IN ('yes','no')),
  CONSTRAINT enumAccessEditor        CHECK (AccessEditor      IN ('yes','no')),
  CONSTRAINT enumAccessJanitor       CHECK (AccessJanitor     IN ('yes','no'))
);

ALTER TABLE ethereal.GalleryAccess         OWNER TO ethereal;

CREATE OR REPLACE FUNCTION ethereal.trigPuppet () RETURNS TRIGGER AS '
  DECLARE
    varPuppet VARCHAR(30);				-- Puppet Default

  BEGIN

    --
    -- REMOVE
    IF TG_OP = ''DELETE'' THEN

      -- Removing root handles
      IF OLD.PuppetPool = ''system'' THEN


         -- Cleanup sattelite tables
         -- Ignore table
         EXECUTE ''DELETE FROM ethereal.PuppetIgnore ''
            || '' WHERE PuppetName = ''
            || QUOTE_LITERAL(OLD.PuppetName);


         -- Determine default handle
         SELECT INTO varPuppet * FROM ethereal.getPuppet(OLD.PuppeteerLogin, OLD.PuppetName);



         -- Update tables
         -- Something to replace with
         IF varPuppet IS NOT NULL THEN

            -- Board
            EXECUTE ''UPDATE ethereal.Board SET PuppetName = ''
               || QUOTE_LITERAL(varPuppet)
               || '' WHERE PuppetName = ''
               || QuOTE_LITERAL(OLD.PuppetName);

            -- Comment
            EXECUTE ''UPDATE ethereal.Comment SET PuppetName = ''
               || QUOTE_LITERAL(varPuppet)
               || '' WHERE PuppetName = ''
               || QuOTE_LITERAL(OLD.PuppetName);

            -- Gallery
            EXECUTE ''UPDATE ethereal.Gallery SET GalleryPenName = ''
               || QUOTE_LITERAL(varPuppet)
               || '' WHERE GalleryPenName = ''
               || QuOTE_LITERAL(OLD.PuppetName);

            -- Gallery Access
            EXECUTE ''UPDATE ethereal.GalleryAccess SET PuppetName = ''
               || QUOTE_LITERAL(varPuppet)
               || '' WHERE Puppetname = ''
               || QuOTE_LITERAL(OLD.PuppetName);

            -- Weblog
            EXECUTE ''UPDATE ethereal.Weblog SET PuppetName = ''
               || QUOTE_LITERAL(varPuppet)
               || '' WHERE PuppetName = ''
               || QuOTE_LITERAL(OLD.PuppetName);


         -- Time to remove
         ELSE

            -- Board
            EXECUTE ''DELETE FROM ethereal.Board ''
               || '' WHERE PuppetName = ''
               || QUOTE_LITERAL(OLD.PuppetName);

            -- Comment
            EXECUTE ''DELETE FROM ethereal.Comment ''
               || '' WHERE PuppetName = ''
               || QUOTE_LITERAL(OLD.PuppetName);

            -- Gallery
            EXECUTE ''DELETE FROM ethereal.Gallery ''
               || '' WHERE GalleryPenName = ''
               || QUOTE_LITERAL(OLD.PuppetName);

            -- Gallery
            EXECUTE ''DELETE FROM ethereal.GalleryAccess ''
               || '' WHERE PuppetName = ''
               || QUOTE_LITERAL(OLD.PuppetName);

            -- Weblog
            EXECUTE ''DELETE FROM ethereal.Weblog ''
               || '' WHERE PuppetName = ''
               || QUOTE_LITERAL(OLD.PuppetName);

         END IF;


      END IF;


      -- Return row unmodified
      RETURN OLD;


    --
    -- UPDATE
    ELSIF TG_OP = ''UPDATE'' THEN

      -- Handle rename
      IF OLD.PuppetName <> NEW.PuppetName AND NEW.PuppetPool = ''system'' THEN

         -- Board
         EXECUTE ''UPDATE ethereal.Board SET PuppetName = ''
            || QUOTE_LITERAL(NEW.PuppetName)
            || '' WHERE PuppetName = ''
            || QuOTE_LITERAL(OLD.PuppetName);

         -- Comment
         EXECUTE ''UPDATE ethereal.Comment SET PuppetName = ''
            || QUOTE_LITERAL(NEW.PuppetName)
            || '' WHERE PuppetName = ''
            || QuOTE_LITERAL(OLD.PuppetName);

         -- Gallery
         EXECUTE ''UPDATE ethereal.Gallery SET GalleryPenName = ''
            || QUOTE_LITERAL(NEW.PuppetName)
            || '' WHERE GalleryPenName = ''
            || QuOTE_LITERAL(OLD.PuppetName);

         -- Gallery Access
         EXECUTE ''UPDATE ethereal.GalleryAccess SET PuppetName = ''
            || QUOTE_LITERAL(NEW.PuppetName)
            || '' WHERE Puppetname = ''
            || QuOTE_LITERAL(OLD.PuppetName);

         -- Puppet Ignore
         EXECUTE ''UPDATE ethereal.PuppetIgnore SET PuppetName = ''
            || QUOTE_LITERAL(NEW.PuppetName)
            || '' WHERE PuppetName = ''
            || QuOTE_LITERAL(OLD.PuppetName);

         -- Weblog
         EXECUTE ''UPDATE ethereal.Weblog SET PuppetName = ''
            || QUOTE_LITERAL(NEW.PuppetName)
            || '' WHERE PuppetName = ''
            || QuOTE_LITERAL(OLD.PuppetName);


      -- Banned handles
      ELSIF NEW.PuppetPool = ''banned'' AND OLD.PuppetPool <> ''banned'' THEN

         -- Board
         EXECUTE ''DELETE FROM ethereal.Board ''
            || '' WHERE PuppetName = ''
            || QUOTE_LITERAL(OLD.PuppetName);

         -- Comment
         EXECUTE ''DELETE FROM ethereal.Comment ''
            || '' WHERE PuppetName = ''
            || QUOTE_LITERAL(OLD.PuppetName);

         -- Gallery
         EXECUTE ''DELETE FROM ethereal.Gallery ''
            || '' WHERE GalleryPenName = ''
            || QUOTE_LITERAL(OLD.PuppetName);

         -- Gallery Access
         EXECUTE ''DELETE FROM ethereal.GalleryAccess ''
            || '' WHERE PuppetName = ''
            || QUOTE_LITERAL(OLD.PuppetName);

         -- Weblog
         EXECUTE ''DELETE FROM ethereal.Weblog ''
            || '' WHERE PuppetName = ''
            || QUOTE_LITERAL(OLD.PuppetName);
      END IF;

      -- Return row unmodified
      RETURN NEW;

    END IF;


  END;
 ' LANGUAGE 'plpgsql';


--
--
-- NAME        : Revision
-- REFERENCES  : Work
--
-- DESCRIPTION : Revisions exist for text base galleries only and reflect revisions
--               made to specific works.

CREATE TABLE ethereal.Revision (
  GalleryID            INT           NOT NULL,
  WorkID               INT           NOT NULL,
  RevisionTimestamp    TIMESTAMP     NOT NULL DEFAULT LOCALTIMESTAMP,
  PuppeteerLogin       VARCHAR(10)   NOT NULL,
  RevisionName         VARCHAR(30)   NOT NULL,
  RevisionText         TEXT,
  CONSTRAINT pkRevision              PRIMARY KEY (GalleryID, WorkID, RevisionTimestamp),
  CONSTRAINT fkRevision_Work         FOREIGN KEY (GalleryID, WorkID)
    REFERENCES ethereal.Work (GalleryID, WorkID)
      ON DELETE CASCADE
      ON UPDATE CASCADE,
  CONSTRAINT fkRevision_Owner        FOREIGN KEY (PuppeteerLogin)
    REFERENCES ethereal.Puppeteer (PuppeteerLogin)
      ON DELETE CASCADE
      ON UPDATE CASCADE
) WITHOUT OIDS;

ALTER TABLE ethereal.Revision              OWNER TO ethereal;



CREATE SEQUENCE ethereal.seqOffline     MAXVALUE 2147483647 CYCLE;

--
--
-- NAME        : Offline
-- REFERENCES  : Realm*
--               Puppet*
--               PuppeteerLogin*
--
-- DESCRIPTION : Offline like post is used to store activity generated in realms.  The
--               difference being that these are meant to remain in place for much
--               longer periods of time.   Similiar to message boards. 

CREATE TABLE ethereal.Offline (
  OfflineIDNumber        INT           NOT NULL DEFAULT NEXTVAL('ethereal.seqOffline'),
  RealmName              VARCHAR(30)   NOT NULL,
  PuppetName             VARCHAR(30),
  PuppeteerLogin         VARCHAR(10),
  OfflineTimestamp       TIMESTAMP     NOT NULL DEFAULT LOCALTIMESTAMP,
  OfflineFullFormat      TEXT,
  OfflineImagelessFormat TEXT,
  OfflinePartialFormat   TEXT,
  CONSTRAINT pkOffline                 PRIMARY KEY (OfflineIDNumber)
) WITHOUT OIDS;

CREATE INDEX idxOffline_Login ON ethereal.Offline
(
  PuppeteerLogin
);

CREATE INDEX idxOffline_Puppet ON ethereal.Offline
(
  PuppetName
);

CREATE INDEX idxOffline_Realm ON ethereal.Offline
(
  RealmName
);

CREATE INDEX idxOffline_Timestamp ON ethereal.Offline
(
  OfflineTimestamp
);

-- MULTI COLUMN
CREATE INDEX idxOffline_NumberRealm ON ethereal.Offline
(
  OfflineIDNumber,
  RealmName
);

CREATE INDEX idxOffline_TimestampPuppet ON ethereal.Offline
(
  OfflineTimestamp,
  PuppetName
);


-- SEQUENCES
ALTER TABLE ethereal.seqOffline            OWNER TO ethereal;

-- TABLES
ALTER TABLE ethereal.Offline               OWNER TO ethereal;

-- CONTRAINTS
ALTER TABLE ethereal.Post  DROP CONSTRAINT enumPostClass;
ALTER TABLE ethereal.Post  ADD CONSTRAINT enumPostClass CHECK (PostCLass IN ('banner','dice','duplicate','general','offline','play','private','special','system'));

--
--
-- NAME        : Watch
-- REFERENCES  : Puppet
--               Puppeteer
--               Realm
--
-- DESCRIPTION : When a new post is made to offline realms, users have the ability to
--               request being notified of these changes.    This table tracks the
--               desire to do so.
--
--               Normally referential integrity of puppets are done through the
--               stored proceedure.  This table is different due to privacy
--               concerns.

CREATE TABLE ethereal.Watch (
  PuppetName           VARCHAR(30)   NOT NULL,
  PuppetPool           VARCHAR(30)   NOT NULL,
  RealmName            VARCHAR(30)   NOT NULL,
  PuppeteerLogin       VARCHAR(10)   NOT NULL,
  CONSTRAINT pkWatch                 PRIMARY KEY (PuppetName, PuppetPool, RealmName, PuppeteerLogin),
  CONSTRAINT fkWatch_Puppet          FOREIGN KEY (PuppetName, PuppetPool)
    REFERENCES ethereal.Puppet (PuppetName, PuppetPool)
      ON DELETE CASCADE
      ON UPDATE CASCADE,
  CONSTRAINT fkWatch_Realm           FOREIGN KEY (RealmName)
    REFERENCES ethereal.Realm (RealmName)
      ON DELETE CASCADE
      ON UPDATE CASCADE,
  CONSTRAINT fkWarch_Owner           FOREIGN KEY (PuppeteerLogin)
    REFERENCES ethereal.Puppeteer (PuppeteerLogin)
      ON DELETE CASCADE
      ON UPDATE CASCADE
) WITHOUT OIDS;

ALTER TABLE ethereal.Work              OWNER TO ethereal;



-- Move Certain Tags
UPDATE ethereal.Tag SET TagType='configuration' WHERE TagName='TagPasswordOld' AND TagType='login';
UPDATE ethereal.Tag SET TagType='configuration' WHERE TagName='TagEMail' AND TagType='login';
UPDATE ethereal.Tag SET TagType='configuration' WHERE TagName='TxtSubmitButtonSub' AND TagType='login';
UPDATE ethereal.Tag SET TagType='configuration' WHERE TagName='OptJavascript' AND TagType='login';
UPDATE ethereal.Tag SET TagType='configuration' WHERE TagName='OptChatPref' AND TagType='login';
UPDATE ethereal.Tag SET TagType='configuration' WHERE TagName='OptFormat' AND TagType='login';
UPDATE ethereal.Tag SET TagType='configuration' WHERE TagName='OptPrivacy' AND TagType='login';
UPDATE ethereal.Tag SET TagType='configuration' WHERE TagName='TagOptJavascript' AND TagType='login';
UPDATE ethereal.Tag SET TagType='configuration' WHERE TagName='TagOptChatPref' AND TagType='login';
UPDATE ethereal.Tag SET TagType='configuration' WHERE TagName='TagOptFormat' AND TagType='login';
UPDATE ethereal.Tag SET TagType='configuration' WHERE TagName='TagOptIgnore' AND TagType='login';
UPDATE ethereal.Tag SET TagType='configuration' WHERE TagName='TagOptPrivacy' AND TagType='login';
UPDATE ethereal.Tag SET TagType='configuration' WHERE TagName='TagOptIgnore' AND TagType='login';
UPDATE ethereal.Tag SET TagType='configuration' WHERE TagName='TxtConfirm' AND TagType='login';

UPDATE ethereal.Tag SET TagName='LnkBoard' WHERE TagName='TagLnkBoard' AND TagType='board';
UPDATE ethereal.Tag SET TagName='LnkBio' WHERE TagName='TagLnkBio' AND TagType='board';



-- Injection code
ALTER TABLE ethereal.Gummy DROP CONSTRAINT enumGummyClass;
ALTER TABLE ethereal.Gummy ADD  CONSTRAINT enumGummyClass CHECK (GummyClass IN ('admin','default','canon','contributor','general','inject','mod','super'));
ALTER TABLE ethereal.Gummy ADD  COLUMN GummyMacro VARCHAR(10);

-- Gummy viewer enhancements
ALTER TABLE ethereal.Tag   DROP CONSTRAINT enumTagType;
ALTER TABLE ethereal.Tag   ADD  CONSTRAINT enumTagType CHECK (TagType IN ('admin','board','configuration','contrib','faq','gallery','gummy','home','login','mail','poll','portal','realm','system','template','vcodes'));

CREATE INDEX idxGummy_Macro ON ethereal.Gummy
(
       GummyMacro
);


DELETE FROM ethereal.Tag WHERE TagType='board'
AND TagName IN ('TagRealm',
                'TagTopic',
                'TmplStatic',
                'TmplMain',
                'TmplSub',
                'TmplMain');

--
-- INTERNAL MAIL REDIRECTION
--
ALTER TABLE ethereal.Puppeteer ADD PuppeteerMailer CHAR(3);
UPDATE ethereal.Puppeteer  SET PuppeteerMailer='int';
ALTER TABLE ethereal.Puppeteer  ALTER COLUMN PuppeteerMailer SET NOT NULL;
ALTER TABLE ethereal.Puppeteer  ALTER COLUMN PuppeteerMailer SET DEFAULT 'int';
ALTER TABLE ethereal.Puppeteer  ADD CONSTRAINT enumPuppeteerMailer CHECK (PuppeteerMailer IN ('int','ext'));

-- Drop old table
DROP TABLE ethereal.Mail;

CREATE TABLE ethereal.Mail (
  MailIDNumber         INT           NOT NULL DEFAULT NEXTVAL('ethereal.seqMail'),
  PuppeteerLogin       VARCHAR(10)   NOT NULL,
  MailFrom             VARCHAR(30),
  MailRcpt             VARCHAR(30),
  MailSubject          VARCHAR(250),
  MailTimestamp        TIMESTAMP     NOT NULL DEFAULT LOCALTIMESTAMP,
  MailRead             TIMESTAMP,
  MailContents         TEXT,
  CONSTRAINT pkMail                  PRIMARY KEY (MailIDNumber)
) WITHOUT OIDS;

-- STANDARD INDEX
CREATE INDEX idxMail_Login ON  ethereal.Mail
(
       PuppeteerLogin
);

CREATE INDEX idxMail_Timestamp ON  ethereal.Mail
(
       MailTimestamp
);

--
-- PUBLIC SITE MANAGEMENT ADDITION
--
ALTER TABLE public.Site ADD SiteRoot VARCHAR(255);
UPDATE public.Site      SET SiteRoot='/tmp';
ALTER TABLE public.Site ALTER COLUMN SiteRoot SET NOT NULL;

-- NAME        : Exif
-- REFERENCES  : NIL
--
-- DESCRIPTION : Allows to fine tune and manage EXIF tags pulled from images for
--               gallery use.

CREATE TABLE ethereal.Exif (
  ExifTag              VARCHAR(20)   NOT NULL,
  ExifMacro            VARCHAR(10)   NOT NULL,
  CONSTRAINT pkExif                  PRIMARY KEY (ExifTag)
) WITHOUT OIDS;

--
-- GALLERY ADJUSTMENTS
ALTER TABLE ethereal.Work ADD COLUMN WorkImageView VARCHAR(30);
ALTER TABLE ethereal.Work ADD COLUMN WorkImageFull VARCHAR(30);
ALTER TABLE ethereal.Work ADD COLUMN WorkImageExif TEXT;


--
--
-- NAME        : Index
-- REFERENCES  : Gallery
--               Work
--
-- DESCRIPTION : This table exists to create a reverse index for all of the
--               works on the site.   Once populated creates the index useable
--               for search engine queries.

CREATE TABLE ethereal.Index (
  IndexWord            VARCHAR(50)   NOT NULL,
  GalleryID            INT           NOT NULL,
  WorkID               INT           NOT NULL,
  IndexScore           SMALLINT,
  CONSTRAINT pkIndex                 PRIMARY KEY (IndexWord, GalleryID, WorkID),
  CONSTRAINT fkIndex                 FOREIGN KEY (GalleryID, WorkID)
    REFERENCES ethereal.Work (GalleryID, WorkID)
      ON DELETE CASCADE
      ON UPDATE CASCADE
) WITHOUT OIDS;


--
-- LAYER HANDLING
--
CREATE SEQUENCE ethereal.seqLayer    MAXVALUE 2147483647;

CREATE TABLE ethereal.Layer (
  LayerID              INT           NOT NULL DEFAULT NEXTVAL('ethereal.seqLayer'),
  GalleryID            INT           NOT NULL,
  LayerShort           VARCHAR(16),
  LayerFull            VARCHAR(100),
  LayerDescription     VARCHAR(255),
  LayerHighlight       VARCHAR(30),
  LayerThumbnail       VARCHAR(200),
  LayerLevel           SMALLINT      NOT NULL DEFAULT '0',
  LevelZero            INT           NOT NULL DEFAULT '0',
  LevelOne             INT           NOT NULL DEFAULT '0',
  LevelTwo             INT           NOT NULL DEFAULT '0',
  LevelThree           INT           NOT NULL DEFAULT '0',
  LevelFour            INT           NOT NULL DEFAULT '0',
  LevelFive            INT           NOT NULL DEFAULT '0',
  CONSTRAINT pkLayer                 PRIMARY KEY (LayerID, GalleryID),
  CONSTRAINT fkLayer                 FOREIGN KEY (GalleryID)
    REFERENCES ethereal.Gallery (GalleryID)
      ON DELETE CASCADE
      ON UPDATE CASCADE
) WITHOUT OIDS;

CREATE INDEX idxLayer_Levels ON ethereal.Layer
(
  LevelZero, LevelOne, LevelTwo, LevelThree, LevelFour, LevelFive
);

ALTER TABLE ethereal.Work ADD COLUMN WorkLayer INT;
ALTER TABLE ethereal.Gallery ADD COLUMN GalleryHighlight VARCHAR(30);
ALTER TABLE ethereal.Gallery ADD COLUMN GalleryThumbnail VARCHAR(200);


--
--
-- TABLE       : Work
--
-- OPERATION   : DELETE
--               UPDATE
--
-- DESCRIPTION : This trigger will ensure that highlighted galleries are proper
--               maintained even as the user changes things around.

CREATE OR REPLACE FUNCTION ethereal.trigWork () RETURNS TRIGGER AS '
  BEGIN

    --
    -- REMOVE
    IF TG_OP = ''DELETE'' THEN

       -- Make sure nothing is highlighted
       IF OLD.WorkLayer IS NOT NULL THEN

          -- Update Layers
          EXECUTE ''UPDATE ethereal.Gallery''
               || '' SET GalleryHighlight=NULL,''
               || ''     GalleryThumbnail=NULL''
               || '' WHERE GalleryID = '' || QUOTE_LITERAL(OLD.GalleryID)
               || ''   AND LayerID = '' || QUOTE_LITERAL(OLD.WorkLayer);

       END IF;

       -- Update Gallery
       EXECUTE ''UPDATE ethereal.Gallery''
            || '' SET GalleryHighlight=NULL,''
            || ''     GalleryThumbnail=NULL''
            || '' WHERE GalleryID = '' || QUOTE_LITERAL(OLD.GalleryID);


      -- Return row unmodified
      RETURN OLD;


    --
    -- UPDATE
    ELSIF TG_OP = ''UPDATE'' THEN

       -- This is a child realm
       IF NEW.WorkName <> OLD.WorkName OR NEW.WorkImageThumbnail <> OLD.WorkImageThumbnail THEN

           -- Make sure nothing is highlighted
           IF OLD.WorkLayer IS NOT NULL AND NEW.WorkLayer IS NOT NULL THEN

              -- Update Layers
              EXECUTE ''UPDATE ethereal.Layer''
                   || '' SET LayerHighlight='' || QUOTE_LITERAL(NEW.WorkName) || '',''
                   || ''     LayerThumbnail='' || QUOTE_LITERAL(NEW.WorkImageThumbnail)
                   || '' WHERE LayerHighlight = '' || QUOTE_LITERAL(OLD.WorkName)
                   || ''   AND LayerThumbnail  = '' || QUOTE_LITERAL(OLD.WorkImageThumbnail)
                   || ''   AND LayerID = '' || QUOTE_LITERAL(NEW.WorkLayer)
                   || ''   AND GalleryID = '' || QUOTE_LITERAL(NEW.GalleryID);

           END IF;

           -- Update Gallery
           EXECUTE ''UPDATE ethereal.Gallery''
                || '' SET GalleryHighlight = '' || QUOTE_LITERAL(NEW.WorkName) || '',''
                || ''     GalleryThumbnail = '' || QUOTE_LITERAL(NEW.WorkImageThumbnail)
                || '' WHERE GalleryHighlight = '' || QUOTE_LITERAL(OLD.WorkName)
                || ''   AND GalleryThumbnail  = '' || QUOTE_LITERAL(OLD.WorkImageThumbnail)
                || ''   AND GalleryID = '' || QUOTE_LITERAL(NEW.GalleryID);
  
       END IF;


       -- Just pass it along
       RETURN NEW;

    END IF;


  END;
 ' LANGUAGE 'plpgsql';


-- ================================================================================== --
-- TRIGGERS                                                                           --
-- ================================================================================== --

-- CREATE TRIGGER
CREATE TRIGGER trig_on_Work
  BEFORE UPDATE OR DELETE
  ON ethereal.Work
  FOR EACH ROW EXECUTE PROCEDURE ethereal.trigWork();


--
-- Genre Enhancements
--
DROP TABLE ethereal.Genre CASCADE;
DROP TABLE ethereal.GalleryGenre CASCADE;

CREATE TABLE ethereal.Genre (
  GenreName            VARCHAR(25)  NOT NULL,
  GenreType            VARCHAR(5)   NOT NULL DEFAULT 'realm',
  GenreSub             VARCHAR(25)  NOT NULL DEFAULT '<root>',
  GenreDescription     VARCHAR(250),
  CONSTRAINT pkGenre                PRIMARY KEY (GenreName, GenreType, GenreSub),
  CONSTRAINT enumGenreOrder         CHECK (GenreType IN ('realm','image','text'))
) WITHOUT OIDS;


CREATE TABLE ethereal.GalleryGenre (
  GalleryID            INT          NOT NULL,
  GenreName            VARCHAR(25)  NOT NULL,
  GenreType            VARCHAR(7)   NOT NULL,
  GenreSub             VARCHAR(25),
  CONSTRAINT pkGalleryGenre         PRIMARY KEY (GalleryID, GenreName, GenreType),
  CONSTRAINT fkGenre_Gallery        FOREIGN KEY (GalleryID)
    REFERENCES ethereal.Gallery (GalleryID)
      ON DELETE CASCADE
      ON UPDATE CASCADE,
  CONSTRAINT fkGenre_Genre          FOREIGN KEY (GenreName, GenreType, GenreSub)
    REFERENCES ethereal.Genre (GenreName, GenreType, GenreSub)
      ON DELETE CASCADE
      ON UPDATE CASCADE
) WITHOUT OIDS;


CREATE VIEW ethereal.Exhibition AS
   SELECT
     gal.GalleryID          AS GalleryID,
     gal.PuppeteerLogin     AS PuppeteerLogin,
     gal.GalleryPenName     AS GalleryPenName,
     gal.GalleryName        AS GalleryName,
     gal.GalleryRating      AS GalleryRating,
     gal.GalleryDescription AS GalleryDescription,
     gal.GalleryType        AS GalleryType,
     gal.GallerySub         AS GallerySub,
     gre.GenreName          AS GalleryGenre,
     gal.GalleryCreated     AS GalleryCreated,
     gal.GalleryUpdated     AS GalleryUpdated,
     (COUNT(wrk.WorkID)-2)  AS WorkCount
   FROM ethereal.Gallery gal, ethereal.Work wrk, ethereal.GalleryGenre gre
   WHERE gal.GalleryID = wrk.GalleryID
     AND gal.GalleryID = gre.GalleryID
     AND gal.GalleryPrivacy = 'no'
   GROUP BY gal.GalleryID,
            gal.PuppeteerLogin,
            gal.GalleryPenName,
            gal.GalleryName,
            gal.GalleryRating,
            gal.GalleryDescription,
            gal.GalleryType,
            gal.GallerySub,
            gal.GalleryCreated,
            gal.GalleryUpdated,
            gre.GenreName
   HAVING COUNT(wrk.WorkID) > 2::INT;


-- Awareness of sub groups --
ALTER TABLE ethereal.Gallery ADD COLUMN GallerySub VARCHAR(25);

-- Addition of a rotation zone
ALTER TABLE ethereal.Tag DROP CONSTRAINT enumTagType;
ALTER TABLE ethereal.Tag ADD  CONSTRAINT enumTagType CHECK (TagType IN ('admin','board','bio','configuration','contrib','faq','gallery','gummy','login','mail','poll','portal','realm','system','template','user','vcodes'));






